@@ -50,21 +50,27 @@ def load_cookies_firefox(cookiejar, profile=None, container=None, domain=None):
 
         sql = ("SELECT name, value, host, path, isSecure, expiry "
                "FROM moz_cookies")
-        parameters = ()
+        conditions = []
+        parameters = []
 
         if container_id is False:
-            sql += " WHERE NOT INSTR(originAttributes,'userContextId=')"
+            conditions.append("NOT INSTR(originAttributes,'userContextId=')")
         elif container_id:
-            sql += " WHERE originAttributes LIKE ? OR originAttributes LIKE ?"
+            conditions.append(
+                "originAttributes LIKE ? OR originAttributes LIKE ?")
             uid = "%userContextId={}".format(container_id)
-            parameters = (uid, uid + "&%")
-        elif domain:
+            parameters += (uid, uid + "&%")
+
+        if domain:
             if domain[0] == ".":
-                sql += " WHERE host == ? OR host LIKE ?"
-                parameters = (domain[1:], "%" + domain)
+                conditions.append("host == ? OR host LIKE ?")
+                parameters += (domain[1:], "%" + domain)
             else:
-                sql += " WHERE host == ? OR host == ?"
-                parameters = (domain, "." + domain)
+                conditions.append("host == ? OR host == ?")
+                parameters += (domain, "." + domain)
+
+        if conditions:
+            sql = "{} WHERE ( {} )".format(sql, " ) AND ( ".join(conditions))
 
         set_cookie = cookiejar.set_cookie
         for name, value, domain, path, secure, expires in db.execute(
@@ -179,11 +185,14 @@ def _firefox_cookies_database(profile=None, container=None):
                                 "{}".format(search_root))
     _log_debug("Extracting cookies from %s", path)
 
-    if container == "none":
+    if not container or container == "none":
         container_id = False
         _log_debug("Only loading cookies not belonging to any container")
 
-    elif container:
+    elif container == "all":
+        container_id = None
+
+    else:
         containers_path = os.path.join(
             os.path.dirname(path), "containers.json")
 
@@ -207,8 +216,6 @@ def _firefox_cookies_database(profile=None, container=None):
                 container))
         _log_debug("Only loading cookies from container '%s' (ID %s)",
                    container, container_id)
-    else:
-        container_id = None
 
     return path, container_id
 
