@@ -23,6 +23,7 @@ modules = [
     "8muses",
     "adultempire",
     "agnph",
+    "ao3",
     "architizer",
     "artstation",
     "aryion",
@@ -31,10 +32,13 @@ modules = [
     "behance",
     "blogger",
     "bluesky",
+    "boosty",
     "bunkr",
     "catbox",
     "chevereto",
     "cien",
+    "civitai",
+    "cohost",
     "comicvine",
     "cyberdrop",
     "danbooru",
@@ -130,7 +134,6 @@ modules = [
     "pornhub",
     "pornpics",
     "postmill",
-    "pururin",
     "reactor",
     "readcomiconline",
     "reddit",
